# frozen_string_literal: true

# Check a user's access to perform a git action. All public methods in this
# class return an instance of `GitlabAccessStatus`
module Gitlab
  class HgAccess < GitAccess
    extend ::Gitlab::Utils::Override

    # This rewrapping of GitAccess exceptions to
    # - start subclassing GitAccess without chasing all places where
    #   an HgAccess exception is been used
    # - keep the possibility that we'd want an exception to become specific
    #   down the road
    # - it's just more natural for Mercurial specific code to check a HgAccess
    #   exception, everything less would look like a bug, that we'd want to
    #   investigate for suspicion of security hole
    ForbiddenError = GitAccess::ForbiddenError
    NotFoundError = GitAccess::NotFoundError
    TimeoutError = GitAccess::TimeoutError
    ProjectMovedError = GitAccess::ProjectMovedError

    # To ease comparison / completion, this is taken from
    #  - mercurial/wireprotov1server.py (5.0.1)
    #  - evolve v9.0.0 (only has obshashranges, and no former obsolete command)
    # ordering of each of these sources has been carried over to facilitate
    # updates.
    # NB: 'hello' seems to be for SSH only, still included not to lose time
    # with it
    DOWNLOAD_COMMANDS = %w{batch between branchmap branches clonebundles capabilities changegroup changegroupsubset debugwireargs getbundle getlfile statlfile heads hello listkeys lookup known protocaps stream_out evoext_obshashrange_v1}.freeze
    PUSH_COMMANDS = %w{unbundle pushkey putlfile}.freeze
    ALL_COMMANDS = DOWNLOAD_COMMANDS + PUSH_COMMANDS

    def initialize(actor, container, protocol, accept_mr_iid: nil, **named_args)
      super(actor, container, protocol, **named_args)
      @accept_mr_iid = accept_mr_iid
    end

    # This method is for cases the command is not known upfront
    # typically SSH, because it is inside the protocol, hence obscure to
    # GitLab components
    #
    # Since we can't use the value of `@cmd`, the checks done in
    # check(cmd, changes) that actually use `@cmd` have to be translated.
    # Here's the summary, to ease further maintaining this security-critical
    # code path:
    #
    # - check_authentication_abilities!: refusing write or read commands
    #                                    according to abilitites
    #       converted in read or write access_level according to abilities
    # - check_command_disabled!: refusing write or read commands over HTTP
    #       converted in read or write access_level accordingly
    # - check_command_existence!
    #       not relevant for Mercurial, the underlying hg process will refuse
    #       any unknown command (it is a process argument, not a process name)
    # - check_db_accessibility!: refuse write commands if db is read-only
    #       converted by returning false in hg_user_can_push?
    # - ensure_project_on_push!(changes): TODO
    def access_level
      @logger = Checks::TimedLogger.new(timeout: INTERNAL_TIMEOUT, header: LOG_HEADER)
      check_protocol!
      check_valid_actor!
      check_active_user!
      check_container!

      # check_otp_session! removed in base class by jump from GitLab 13.7 to 13.9

      check_repository_existence!
      access_level_early_checks_ok
    end

    # Separated method for cases where the first checks are already
    # done (HTTP controller)
    def access_level_early_checks_ok
      if hg_user_can_push?
        # takes into account transversal stuff, such as temporary readonly etc.
        # these should apply even to users with publication rights
        if user_allowed_to_push_can_also_publish?
          'publish'
        else
          'write'
        end
      elsif can_read_project?
        if (@protocol == 'http') && upload_pack_disabled_over_http?
          nil
        elsif !(authentication_abilities.include?(:download_code) || authentication_abilities.include?(:build_download_code))
          nil
        else
          'read'
        end
      else
        nil
      end
    end

    # Does a user that is already allowed to push also publish?
    def user_allowed_to_push_can_also_publish?
      # this is a good approximation, after a quick browse, this seems to
      # be also what's checked by GitLab to see if user can push to a
      # protected branch
      # TODO, in heptapod branch, use the new hg_can_publish? method
      user_access.can_do_action?(:admin_project)
    end

    # high level push permission check. This includes in particular
    # shortcuts for deploy keys and the like.
    def hg_user_can_push?
      return false if @protocol == 'http' && receive_pack_disabled_over_http?
      return false if Gitlab::Database.read_only?
      return false unless authentication_abilities.include?(:push_code)

      @changes = ANY
      check_push_access!
      true
    rescue ForbiddenError, Gitlab::GitAccess::ForbiddenError
      false
    end

    override :download_ability
    def download_ability
      :download_code
    end

    override :push_ability
    def push_ability
      :push_code
    end

    # The base class uses this in check_command_disabled!
    # to enforce protocol based access control
    # let's convert to something meaningful for Mercurial
    def upload_pack?
      DOWNLOAD_COMMANDS.include?(cmd)
    end

    # Same explanation as with upload_pack?
    def receive_pack?
      PUSH_COMMANDS.include?(cmd)
    end

    def hg_access?
      true
    end
  end
end
