# frozen_string_literal: true

require "spec_helper"

# rubocop:disable RSpec/FilePath
RSpec.describe "Blob and Tree access HgitalyRepository", :seed_helper do
  include Gitlab::EncodingHelper
  include RepoHelpers
  include Gitlab::Popen

  def skip_if_hgitaly_no_blob_tree
    version = RSpec.configuration.hgitaly_version
    # Gem version scheme understands the 'dev' scheme of Python PEP 440 somewhat:
    # - an alphabetic part always means pre-release and is sorted lexicographically
    # - the number after is taken into account
    # so actually 'dev0' is greater that 'a1' and 'b1'. That's good enough for now, though.
    if version < Gem::Version.new('0.12.0dev1')
      skip "Skipped because HGitaly version #{version} does not implement blob/tree methods yet"
    end
  end

  def hgitaly_repo_init
    repo = Gitlab::Mercurial::HgitalyRepository.new(
      'default', 'hgitaly-repo.git', '', 'group/project')
    repo.unit_tests_skip_hooks = true
    repo
  end

  let(:hgitaly_repo) { hgitaly_repo_init }

  before(:all) do
    repo = hgitaly_repo_init
    status = repo.create_repository(namespace_fs_path: "group", in_subgroup: false)
    expect(status).to be(true)

    user = build(:user)

    repo.multi_action(user,
                      branch_name: 'branch/default',
                      message: "Commit 0",
                      author_email: "jdoe1@heptapod.test",
                      author_name: "John Doe1",
                      actions: [{ action: :create,
                                  file_path: 'foo',
                                  content: "foo0\n" }])
    repo.expire_has_local_branches_cache # we have no post-receive hooks
    repo.multi_action(user,
                      branch_name: 'branch/default',
                      start_branch_name: 'branch/default',
                      message: "Commit 1",
                      author_email: "jdoe1@heptapod.test",
                      author_name: "John Doe1",
                      actions: [{ action: :create,
                                  file_path: "sub/foo1",
                                  content: "foo1\n" }])
  end

  # Same repository as :repository, seen as HgGitRepository, so that
  #
  #   1. we can use HgGitRepository as a reference for assertions on method
  #      results
  #   2. we have easy fixture preparation options
  let(:hg_git_repo) do
    repo = Gitlab::Mercurial::HgGitRepository.new(
      hgitaly_repo.storage,
      hgitaly_repo.relative_path,
      hgitaly_repo.gl_repository,
      hgitaly_repo.gl_project_path)
    repo.unit_tests_skip_hooks = true
    repo
  end

  after(:all) do
    hgitaly_repo_init.remove
  end

  describe 'lfs pointers' do
    it 'returns empty arrays instead of breaking' do
      # we don't know as of this writing how Mercurial LFS will be integrated
      # in Heptapod, maybe we'll go through some other methods, maybe not.
      # Meanwhile there must be no break.
      client = hgitaly_repo.gitaly_blob_client
      sha = hgitaly_repo.find_branch('branch/default').target
      ['branch/default', sha].each do |rev|
        expect(client.get_new_lfs_pointers(rev, nil, nil, nil)).to be_empty
      end
      expect(client.get_all_lfs_pointers).to be_empty
    end
  end

  shared_examples 'Blob/Tree common integration tests' do
    let(:hg_commit_sha) { hgitaly_repo.hgsha_from_rev('default') }
    let(:git_commit_sha) { hg_git_repo.commit.sha }

    it 'HGitalyRepository#blob_at returns Blob instances from a Mercurial Changeset ID' do
      blob = hgitaly_repo.blob_at(hg_commit_sha, 'foo')
      expect(blob.data).to eq("foo0\n")
      expect(blob.commit_id).to eq(hg_commit_sha)

      ref_blob = hgitaly_repo.blob_at('branch/default', 'foo')
      expect(ref_blob.data).to eq(blob.data)

      hg_git_repo_blob = hg_git_repo.blob_at(git_commit_sha, 'foo')
      expect(hg_git_repo_blob.data).to eq("foo0\n")
      expect(hg_git_repo_blob.commit_id).to eq(git_commit_sha)

      blobs = hgitaly_repo.batch_blobs([[hg_commit_sha, 'foo'],
                                        [hg_commit_sha, 'wrong-path']])
      expect(blobs.length).to eq(1)
      expect(blobs_equal?(blobs[0], blob)).to be(true)

      hg_git_repo_blobs = hg_git_repo.batch_blobs(
        [[git_commit_sha, 'foo'],
         [git_commit_sha, 'wrong-path']])
      expect(hg_git_repo_blobs.length).to eq(1)
      expect(blobs_equal?(hg_git_repo_blobs[0], hg_git_repo_blob)).to be(true)
    end

    it 'Tree.tree_entries works with Mercurial Changeset IDs' do
      entries = Gitlab::Git::Tree.tree_entries(hgitaly_repo, hg_commit_sha, '', true)
      expect(entries[0].path).to eq("foo")
      expect(entries[0].type).to be(:blob)
      expect(entries[0].commit_id).to eq(hg_commit_sha)

      expect(entries[1].path).to eq("sub")
      expect(entries[1].type).to be(:tree)
      expect(entries[1].commit_id).to eq(hg_commit_sha)

      expect(entries[2].path).to eq("sub/foo1")
      expect(entries[2].type).to be(:blob)
      expect(entries[2].commit_id).to eq(hg_commit_sha)

      # also works with refs (this seems to be actually used from the Rails app)
      ref_entries = Gitlab::Git::Tree.tree_entries(hgitaly_repo, 'branch/default', '',
                                                   false)
      expect(ref_entries.length).to eq(2) # recurse ^^^^ is false
      expect(ref_entries[0].path).to eq("foo")
      expect(ref_entries[0].type).to be(:blob)
      expect(ref_entries[0].id).to eq(entries[0].id)
      expect(ref_entries[1].path).to eq("sub")
      expect(ref_entries[1].type).to be(:tree)

      # using a returned blob oid just works
      # At the HGitaly1 milestone, this is Gitaly working, here
      # With HGitaly2 it will turn into the kind of blob oids that HGitaly
      # works with.
      blob = hgitaly_repo.gitaly_blob_client.get_blob(oid: entries[2].id,
                                                      limit: 4)
      # consistency with hg_git_repo
      expect(blob.data).to eq('foo1')
      hg_git_entries = Gitlab::Git::Tree.tree_entries(
        hg_git_repo, git_commit_sha, '', true)
      expect(hg_git_entries.length).to eq(3)
      entries.zip(hg_git_entries) do |hg_entry, git_entry|
        expect(hg_entry.path).to eq(git_entry.path)
        expect(hg_entry.type).to eq(git_entry.type)
        # not asserting equality of ids because that's the part which
        # will change in the HGitaly2 milestone
      end
    end

    it 'HGitalyRepository#last_commits_for_tree returns Commits with Mercurial Changeset IDs' do
      # using '0' as hg revspec because it's just convenient to grab the
      # root (first ever done commit) from here
      root_hgsha = hgitaly_repo.hgsha_from_rev('0')
      root_gitsha = hgitaly_repo.gitsha_from_sha(root_hgsha)

      foo_last_commit = hgitaly_repo.list_last_commits_for_tree(
        hg_commit_sha, 'foo')['foo']
      expect(foo_last_commit.id).to eq(root_hgsha)
      expect(foo_last_commit.message.strip).to eq("Commit 0")
      expect(foo_last_commit.instance_variable_get(:@repository))
        .to eq(hgitaly_repo)

      sub_foo1_last_commit = hgitaly_repo.list_last_commits_for_tree(
        hg_commit_sha, 'sub/foo1')['sub/foo1']
      expect(sub_foo1_last_commit.id).to eq(hg_commit_sha)

      # consistency with hg_git_repo
      foo_git_last_commit = hg_git_repo.list_last_commits_for_tree(
        git_commit_sha, 'foo')['foo']
      expect(foo_git_last_commit.id).to eq(root_gitsha)
      expect(foo_git_last_commit.message.strip).to eq("Commit 0")
      expect(foo_git_last_commit.instance_variable_get(:@repository))
        .to eq(hg_git_repo)
      sub_foo1_git_last_commit = hg_git_repo.list_last_commits_for_tree(
        git_commit_sha, 'sub/foo1')['sub/foo1']
      expect(sub_foo1_git_last_commit.id).to eq(git_commit_sha)
    end
  end

  describe 'In HGitaly1 mode' do
    before do
      hgitaly_repo.is_hgitaly1 = true
    end

    it_behaves_like 'Blob/Tree common integration tests'
  end

  describe 'In fully native mode' do
    before do
      skip_if_hgitaly_no_blob_tree
      hgitaly_repo.is_hgitaly1 = false
    end

    it_behaves_like 'Blob/Tree common integration tests'
  end

  # Tell if two blobs have identical instance variables
  #
  # Blobs don't implement equality, and it could possibly be a problem
  # with partially loaded data if they did.
  # Hence `expect(blob1).to eq(blob2)` fails in any case unless they
  # are the very same Ruby objects.
  def blobs_equal?(blob1, blob2)
    %i[id mode name path size commit_id data binary].all? do |attr|
      blob1.send(attr) == blob2.send(attr)
    end
  end
end
